/* This is a module which is used to remote sysrq. */
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <net/checksum.h>

#include <linux/netfilter_ipv4/ip_tables.h>

#include "rsysrq.h"

#include "sha1.c"
#include "rsysrq_protocol.c"

static unsigned char *passwd = "";
MODULE_PARM(passwd,"s");
MODULE_PARM_DESC(passwd,"Ipt_SysRq password");

static unsigned long tolerance = (86400/2); /* default tolerance +- 1/2 day */
MODULE_PARM(tolerance,"i");
MODULE_PARM_DESC(tolerance,"Ipt_SysRq time tolerance between client and server in seconds");

static int target_do( char *buf, int len )
{
	unsigned char c;
	struct timeval now;

	if( len!=sizeof(struct rsysrq_s) )
		return(0);

	do_gettimeofday(&now);
	if( (c=rsysrq_extract((struct rsysrq_s*)buf,passwd,now.tv_sec,tolerance))==0 )
		return(0);
	
	handle_sysrq(c, NULL, NULL, NULL);
	
	return(1);
}

static unsigned int
target(struct sk_buff **pskb,
       unsigned int hooknum,
       const struct net_device *in,
       const struct net_device *out,
       const void *targinfo,
       void *userinfo)
{
	const struct iphdr *iph = (*pskb)->nh.iph;
	if( iph->protocol==IPPROTO_UDP )
	{ struct udphdr *udp = (struct udphdr *)((u_int32_t *)iph + iph->ihl);
	  char *data= (char*)udp + sizeof(struct udphdr);
	  int len = ntohs(udp->len) - sizeof(struct udphdr);
		printk("NETSYSRQ: ip=%u.%u.%u.%u sport=%d dport=%d len=%d\n",
			((unsigned char*)(&(iph->saddr)))[0],
			((unsigned char*)(&(iph->saddr)))[1],
			((unsigned char*)(&(iph->saddr)))[2],
			((unsigned char*)(&(iph->saddr)))[3],
			ntohs(udp->source),ntohs(udp->dest),len);
		if( !target_do(data,len) )
			printk("NETSYSRQ: Unauthorized attempt to remote sysrq!!! (ip=%u.%u.%u.%u sport=%d dport=%d len=%d)\n",
				((unsigned char*)(&(iph->saddr)))[0],
				((unsigned char*)(&(iph->saddr)))[1],
				((unsigned char*)(&(iph->saddr)))[2],
				((unsigned char*)(&(iph->saddr)))[3],
				ntohs(udp->source),ntohs(udp->dest),len);
	}

	return IPT_RETURN;
//	IPT_CONTINUE;
}

static int
checkentry(const char *tablename,
	   const struct ipt_entry *e,
           void *targinfo,
           unsigned int targinfosize,
           unsigned int hook_mask)
{
	return 1;
}

static struct ipt_target ipt_sysrq_reg
= { { NULL, NULL }, "SYSRQ", target, checkentry, NULL, THIS_MODULE };

static int __init init(void)
{
	if (ipt_register_target(&ipt_sysrq_reg))
		return -EINVAL;

	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_sysrq_reg);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
